@@ -1,6 +1,6 @@
 /*
  * ModSecurity for Apache 2.x, http://www.modsecurity.org/
- * Copyright (c) 2004-2007 Breach Security, Inc. (http://www.breach.com/)
+ * Copyright (c) 2004-2008 Breach Security, Inc. (http://www.breach.com/)
  *
  * You should have received a copy of the licence along with this
  * program (stored in the file "LICENSE"). If the file is missing,
@@ -48,7 +48,7 @@ typedef struct msc_string msc_string;
 #include "http_protocol.h"
 
 #define MODULE_NAME "ModSecurity"
-#define MODULE_RELEASE "2.1.3"
+#define MODULE_RELEASE "2.1.7"
 #define MODULE_NAME_FULL (MODULE_NAME " v" MODULE_RELEASE " (Apache 2.x)")
 
 #define PHASE_REQUEST_HEADERS       1
@@ -57,8 +57,8 @@ typedef struct msc_string msc_string;
 #define PHASE_RESPONSE_BODY         4
 #define PHASE_LOGGING               5
 
-#define NOT_SET     -1
-#define NOT_SET_P (void *)-1
+#define NOT_SET                    -1l
+#define NOT_SET_P          (void *)-1l
 
 #define CREATEMODE ( APR_UREAD | APR_UWRITE | APR_GREAD )
 #define CREATEMODE_DIR ( APR_UREAD | APR_UWRITE | APR_UEXECUTE | APR_GREAD | APR_GEXECUTE )
@@ -285,6 +285,7 @@ struct modsec_rec {
     apr_md5_ctx_t        new_auditlog_md5ctx;
 
     unsigned int         was_intercepted;
+    unsigned int         rule_was_intercepted;
     unsigned int         intercept_phase;
     msre_actionset      *intercept_actionset;
     const char          *intercept_message;
@@ -376,6 +377,7 @@ struct directory_config {
     const char          *upload_dir;
     int                  upload_keep_files;
     int                  upload_validates_files;
+    int                  upload_filemode;
 
     /* Used only in the configuration phase. */
     msre_rule           *tmp_chain_starter;
