@@ -1,6 +1,6 @@
 /*
  * ModSecurity for Apache 2.x, http://www.modsecurity.org/
- * Copyright (c) 2004-2007 Breach Security, Inc. (http://www.breach.com/)
+ * Copyright (c) 2004-2008 Breach Security, Inc. (http://www.breach.com/)
  *
  * You should have received a copy of the licence along with this
  * program (stored in the file "LICENSE"). If the file is missing,
@@ -9,6 +9,7 @@
  *
  */
 #include <ctype.h>
+#include <sys/stat.h>
 
 #include "msc_multipart.h"
 #include "msc_util.h"
@@ -377,6 +378,21 @@ static int multipart_process_part_data(modsec_rec *msr, char **error_msg) {
 
                 msr_log(msr, 4, "Multipart: Created temporary file: %s",
                     log_escape_nq(msr->mp, msr->mpd->mpp->tmp_file_name));
+
+                #if (!(defined(WIN32) || defined(NETWARE)))
+                if (msr->txcfg->debuglog_level >= 9) {
+                    msr_log(msr, 9, "Multipart: Changing file mode to %04o: %s", msr->txcfg->upload_filemode, log_escape_nq(msr->mp, msr->mpd->mpp->tmp_file_name));
+                }
+                if (fchmod(msr->mpd->mpp->tmp_file_fd, msr->txcfg->upload_filemode) < 0) {
+                    
+                    char errbuf[256];
+                    if (msr->txcfg->debuglog_level >= 3) {
+                        msr_log(msr, 3, "Multipart: Could not change mode on \"%s\" (%d): %s",
+                            log_escape_nq(msr->mp, msr->mpd->mpp->tmp_file_name),
+                            errno, apr_strerror(APR_FROM_OS_ERROR(errno), errbuf, 256));
+                    }
+                }
+                #endif
             }
 
             /* write the reserve first */
@@ -603,8 +619,11 @@ static int multipart_count_boundary_params(apr_pool_t *mp, const char *header_va
 
     s = duplicate;
     while((s = strstr(s, "boundary")) != NULL) {
-        count++;
         s += 8;
+
+        if (strchr(s, '=') != NULL) {
+            count++;
+        }
     }
 
     return count;
@@ -922,7 +941,7 @@ int multipart_process_chunk(modsec_rec *msr, const char *buf,
                     }
     
                     if (   (p != msr->mpd->buf + 2)
-                        && (strncmp(p, msr->mpd->boundary, strlen(msr->mpd->boundary)))
+                        && (strncmp(p, msr->mpd->boundary, strlen(msr->mpd->boundary)) == 0)
                     ) {
                         /* Found whitespace in front of a boundary. */
                         msr->mpd->flag_error = 1;
@@ -1039,6 +1058,12 @@ apr_status_t multipart_cleanup(modsec_rec *msr) {
         for(i = 0; i < msr->mpd->parts->nelts; i++) {
             if (parts[i]->type == MULTIPART_FILE) {
                 if (parts[i]->tmp_file_name != NULL) {
+                    /* make sure it is closed first */
+                    if (parts[i]->tmp_file_fd > 0) {
+                        close(parts[i]->tmp_file_fd);
+                        parts[i]->tmp_file_fd = -1;
+                    }
+
                     if (unlink(parts[i]->tmp_file_name) < 0) {
                         msr_log(msr, 1, "Multipart: Failed to delete file (part) \"%s\" because %d(%s)",
                             log_escape(msr->mp, parts[i]->tmp_file_name), errno, strerror(errno));
@@ -1059,6 +1084,12 @@ apr_status_t multipart_cleanup(modsec_rec *msr) {
             if ((parts[i]->type == MULTIPART_FILE)&&(parts[i]->tmp_file_size == 0)) {
                 /* Delete empty file. */
                 if (parts[i]->tmp_file_name != NULL) {
+                    /* make sure it is closed first */
+                    if (parts[i]->tmp_file_fd > 0) {
+                        close(parts[i]->tmp_file_fd);
+                        parts[i]->tmp_file_fd = -1;
+                    }
+
                     if (unlink(parts[i]->tmp_file_name) < 0) {
                         msr_log(msr, 1, "Multipart: Failed to delete empty file (part) \"%s\" because %d(%s)",
                             log_escape(msr->mp, parts[i]->tmp_file_name), errno, strerror(errno));
@@ -1073,6 +1104,12 @@ apr_status_t multipart_cleanup(modsec_rec *msr) {
                     const char *new_filename = NULL;
                     const char *new_basename = NULL;
 
+                    /* make sure it is closed first */
+                    if (parts[i]->tmp_file_fd > 0) {
+                        close(parts[i]->tmp_file_fd);
+                        parts[i]->tmp_file_fd = -1;
+                    }
+
                     new_basename = file_basename(msr->mp, parts[i]->tmp_file_name);
                     if (new_basename == NULL) return -1;
                     new_filename = apr_psprintf(msr->mp, "%s/%s", msr->txcfg->upload_dir,
