@@ -1,6 +1,6 @@
 /*
  * ModSecurity for Apache 2.x, http://www.modsecurity.org/
- * Copyright (c) 2004-2007 Breach Security, Inc. (http://www.breach.com/)
+ * Copyright (c) 2004-2008 Breach Security, Inc. (http://www.breach.com/)
  *
  * You should have received a copy of the licence along with this
  * program (stored in the file "LICENSE"). If the file is missing,
@@ -607,6 +607,12 @@ apr_status_t modsecurity_request_body_clear(modsec_rec *msr) {
                         log_escape(msr->msc_reqbody_mp, put_filename));
                 }
             } else {
+                /* make sure it is closed first */
+                if (msr->msc_reqbody_fd > 0) {
+                    close(msr->msc_reqbody_fd);
+                    msr->msc_reqbody_fd = -1;
+                }
+
                 /* We do not want to keep the request body. */
                 if (apr_file_remove(msr->msc_reqbody_filename,
                     msr->msc_reqbody_mp) != APR_SUCCESS)
