@@ -1,6 +1,6 @@
 /*
  * ModSecurity for Apache 2.x, http://www.modsecurity.org/
- * Copyright (c) 2004-2007 Breach Security, Inc. (http://www.breach.com/)
+ * Copyright (c) 2004-2008 Breach Security, Inc. (http://www.breach.com/)
  *
  * You should have received a copy of the licence along with this
  * program (stored in the file "LICENSE"). If the file is missing,
@@ -834,10 +834,6 @@ int ansi_c_sequences_decode_inplace(unsigned char *input, int input_len) {
         if ((input[i] == '\\')&&(i + 1 < input_len)) {
             int c = -1;
         
-            /* ENH Should we handle \c as well?
-             *     See http://www.opengroup.org/onlinepubs/009695399/utilities/printf.html
-             */
-
             switch(input[i + 1]) {
                 case 'a' :
                     c = '\a';
@@ -889,14 +885,11 @@ int ansi_c_sequences_decode_inplace(unsigned char *input, int input_len) {
                     }
                 }
                 else
-                if (isdigit(input[i + 1])) { /* Octal. */
-                    char buf[10];
-                    int j = 0, l = 3;
+                if (ISODIGIT(input[i + 1])) { /* Octal. */
+                    char buf[4];
+                    int j = 0;
 
-                    /* Up to 4 digits if the first digit is a zero. */
-                    if (input[i + 1] == '0') l = 4;
-
-                    while((i + 1 + j < input_len)&&(j <= l)) {
+                    while((i + 1 + j < input_len)&&(j < 3)) {
                         buf[j] = input[i + 1 + j];
                         j++;
                         if (!ISODIGIT(input[i + 1 + j])) break;
