@@ -1,6 +1,6 @@
 /*
  * ModSecurity for Apache 2.x, http://www.modsecurity.org/
- * Copyright (c) 2004-2007 Breach Security, Inc. (http://www.breach.com/)
+ * Copyright (c) 2004-2008 Breach Security, Inc. (http://www.breach.com/)
  *
  * You should have received a copy of the licence along with this
  * program (stored in the file "LICENSE"). If the file is missing,
@@ -161,7 +161,7 @@ apr_table_t *collection_retrieve(modsec_rec *msr, const char *col_name,
                 }
 
                 /* NOTE: No rate if there has been no time elapsed */
-                td = (apr_time_sec(apr_time_now()) - create_time);
+                td = (int)(apr_time_sec(apr_time_now()) - create_time);
                 if (td == 0) {
                     var->value = apr_psprintf(msr->mp, "%i", 0);
                 }
