@@ -1,6 +1,6 @@
 /*
  * ModSecurity for Apache 2.x, http://www.modsecurity.org/
- * Copyright (c) 2004-2007 Breach Security, Inc. (http://www.breach.com/)
+ * Copyright (c) 2004-2008 Breach Security, Inc. (http://www.breach.com/)
  *
  * You should have received a copy of the licence along with this
  * program (stored in the file "LICENSE"). If the file is missing,
@@ -664,6 +664,9 @@ apr_status_t msre_ruleset_process_phase(msre_ruleset *ruleset, modsec_rec *msr)
     for (i = 0; i < arr->nelts; i++) {
         msre_rule *rule = rules[i];
 
+        /* Reset the rule interception flag */
+        msr->rule_was_intercepted = 0;
+
         /* NEXT_CHAIN is used when one of the rules in a chain
          * fails to match and then we need to skip the remaining
          * rules in that chain in order to get to the next
@@ -731,8 +734,8 @@ apr_status_t msre_ruleset_process_phase(msre_ruleset *ruleset, modsec_rec *msr)
         }
         else
         if (rc == RULE_MATCH) {
-            if (msr->was_intercepted) {
-                /* If the transaction was intercepted we will
+            if (msr->rule_was_intercepted) {
+                /* If the transaction was intercepted by this rule we will
                  * go back. Do note that we are relying on the
                  * rule to know if it is a part of a chain and
                  * not intercept if it is.
@@ -922,6 +925,7 @@ int msre_ruleset_rule_remove_with_exception(msre_ruleset *ruleset, rule_exceptio
     count += msre_ruleset_phase_rule_remove_with_exception(ruleset, re, ruleset->phase_request_body);
     count += msre_ruleset_phase_rule_remove_with_exception(ruleset, re, ruleset->phase_response_headers);
     count += msre_ruleset_phase_rule_remove_with_exception(ruleset, re, ruleset->phase_response_body);
+    count += msre_ruleset_phase_rule_remove_with_exception(ruleset, re, ruleset->phase_logging);
 
     return count;    
 }
@@ -1153,6 +1157,7 @@ static void msre_perform_disruptive_actions(modsec_rec *msr, msre_rule *rule,
      * transaction, and rememer the rule that caused it.
      */
     msr->was_intercepted = 1;
+    msr->rule_was_intercepted = 1;
     msr->intercept_phase = msr->phase;
     msr->intercept_actionset = actionset;
     msr->intercept_message = message;
@@ -1389,7 +1394,7 @@ apr_status_t msre_rule_process(msre_rule *rule, modsec_rec *msr) {
                         * was intercepted - no need to process the remaining
                         * targets.
                         */
-                        if (msr->was_intercepted) {
+                        if (msr->rule_was_intercepted) {
                             return RULE_MATCH;
                         }
                     }
@@ -1437,7 +1442,7 @@ apr_status_t msre_rule_process(msre_rule *rule, modsec_rec *msr) {
                  * was intercepted - no need to process the remaining
                  * targets.
                  */
-                if (msr->was_intercepted) {
+                if (msr->rule_was_intercepted) {
                     return RULE_MATCH;
                 }
             }
