@@ -1,6 +1,6 @@
 /*
  * ModSecurity for Apache 2.x, http://www.modsecurity.org/
- * Copyright (c) 2004-2007 Breach Security, Inc. (http://www.breach.com/)
+ * Copyright (c) 2004-2008 Breach Security, Inc. (http://www.breach.com/)
  *
  * You should have received a copy of the licence along with this
  * program (stored in the file "LICENSE"). If the file is missing,
@@ -229,6 +229,9 @@ static int var_rule_generate(modsec_rec *msr, msre_var *var, msre_rule *rule,
 /* ENV */
 
 static char *var_env_validate(msre_ruleset *ruleset, msre_var *var) {
+    if (var->param == NULL) {
+        return apr_psprintf(ruleset->mp, "Parameter required for ENV.");
+    }
     if ((strlen(var->param) > 2)&&(var->param[0] == '/')
         &&(var->param[strlen(var->param) - 1] == '/'))
     {
@@ -1801,7 +1804,7 @@ static int var_multipart_unmatched_boundary_generate(modsec_rec *msr, msre_var *
  */
 static void msre_engine_variable_register(msre_engine *engine, const char *name, 
     unsigned int type, unsigned int argc_min, unsigned int argc_max,
-    FN_VAR_VALIDATE(validate), FN_VAR_GENERATE(generate),
+    fn_var_validate_t validate, fn_var_generate_t generate,
     unsigned int is_cacheable, unsigned int availability)
 {
     msre_var_metadata *metadata = (msre_var_metadata *)apr_pcalloc(engine->mp,
